# -*- coding: utf-8 -*-
from django.test import TestCase


class ManifestViewTest(TestCase):
    urls = "manifesto.urls"

    def test_manifest_render(self):
        response = self.client.get('/manifest.appcache')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'text/cache-manifest')
        self.assertTemplateUsed(response, "manifesto/manifest.appcache")
        context = response.context
        self.assertEqual(context['revision'], '593b743')
        self.assertEqual(context['cache_list'], ['/static/js/application.js',
            '/static/css/screen.css'])
        self.assertEqual(context['network_list'], ['*'])
        self.assertEqual(context['fallback_list'], [('/', '/offline.html')])



import os
local_path = lambda path: os.path.join(os.path.dirname(__file__), path)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'TEST_NAME': ':memory:'
    }
}

ROOT_URLCONF = "manifesto.urls"

SITE_ID = 1

INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'django.contrib.sites',
    'django.contrib.auth',
    'django.contrib.admin',
    'manifesto',
    'tests',
]

SECRET_KEY = "manifesto"



# -*- coding: utf-8 -*-
from django.conf.urls.defaults import *  # noqa

from manifesto.views import ManifestView


urlpatterns = patterns('',
    url(r'^manifest\.appcache$', ManifestView.as_view(), name="cache_manifest"),
)



from manifesto import Manifest


class StaticManifest(Manifest):
  def cache(self):
    return [
      '/static/js/application.js',
      '/static/css/screen.css',
    ]

  def network(self):
    return ['*']

  def fallback(self):
    return [
      ('/', '/offline.html'),
    ]



# -*- coding: utf-8 -*-
from django.views.generic.base import TemplateView

from manifesto import manifest


class ManifestView(TemplateView):
    template_name = "manifesto/manifest.appcache"

    def render_to_response(self, context, **kwargs):
        kwargs['content_type'] = 'text/cache-manifest'
        return super(ManifestView, self).render_to_response(context, **kwargs)

    def get_context_data(self, **kwargs):
        kwargs.update({
            'revision': manifest.revision,
            'cache_list': manifest.cache,
            'network_list': manifest.network,
            'fallback_list': manifest.fallback,
        })
        return kwargs



# -*- coding: utf-8 -*-
from bencode import bencode
import itertools

from django.utils.hashcompat import sha_constructor


class Manifest(object):
    def fallback(self):
        return []

    def network(self):
        return ['*']

    def cache(self):
        return []

    def revision(self):
        revision = list(itertools.chain(self.fallback(), self.network(), self.cache()))
        return sha_constructor(bencode(revision)).hexdigest()[:7]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages


setup(
    name='manifesto',
    version='0.4',
    description='Pluggable cache manifest for Django.',
    author='Timothee Peignier',
    author_email='timothee.peignier@tryphon.org',
    url='https://github.com/cyberdelia/manifesto',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'bencode',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities'
    ],
)


# -*- coding: utf-8 -*-
import inspect

from bencode import bencode

from django.conf import settings
from django.utils import importlib
from django.utils.hashcompat import sha_constructor

from manifesto.manifest import Manifest

__all__ = ['manifest', 'Manifest']


class UnifiedManifest(object):
    def __init__(self):
        self._fallback = []
        self._cache = []
        self._network = []
        self.manifests = []
        self._built = False
        self.excluded_manifests = getattr(settings,
            'MANIFESTO_EXCLUDED_MANIFESTS', [])

    def reset(self):
        self._fallback = []
        self._cache = []
        self._network = []
        self.manifests = []
        self._built = False

    def build(self, manifests=None):
        self.reset()

        if manifests == None:
            manifests = self.collect_manifest()
        self.manifests = manifests

        for manifest in manifests:
            self._fallback += manifest.fallback()
            self._cache += manifest.cache()
            self._network += manifest.network()

        self._built = True

    def collect_manifest(self):
        ignored_classes = [
            Manifest,
        ]
        manifests = []
        for app in settings.INSTALLED_APPS:
            try:
                module = importlib.import_module("%s.manifest" % app)
            except ImportError:
                continue

            for name, item in inspect.getmembers(module, inspect.isclass):
                if not item in ignored_classes and issubclass(item, Manifest):
                    class_path = "%s.manifest.%s" % (app, name)
                    if class_path in self.excluded_manifests:
                        continue
                    manifests.append(item())
        return manifests

    @property
    def fallback(self):
        if not self._built:
            self.build()
        return self._fallback

    @property
    def network(self):
        if not self._built:
            self.build()
        return self._network

    @property
    def cache(self):
        if not self._built:
            self.build()
        return self._cache

    @property
    def revision(self):
        revision = [manifest.revision() for manifest in self.manifests]
        return sha_constructor(bencode(revision)).hexdigest()[:7]

manifest = UnifiedManifest()
